@@ -22,7 +22,6 @@ from dagster.core.definitions.reconstructable import ReconstructablePipeline
 from dagster.core.errors import (
     DagsterInvalidConfigError,
     DagsterInvalidSubsetError,
-    DagsterInvariantViolationError,
     DagsterLaunchFailedError,
     DagsterSubprocessError,
     DagsterUserCodeExecutionError,
@@ -278,7 +277,7 @@ def execution_plan_snapshot_command(args):
     return snapshot_from_execution_plan(
         create_execution_plan(
             pipeline=recon_pipeline,
-            environment_dict=args.run_config,
+            run_config=args.run_config,
             mode=args.mode,
             step_keys_to_execute=args.step_keys_to_execute,
         ),
@@ -314,7 +313,7 @@ def partition_data_command(args):
             lambda: 'Error occurred during the execution of user-provided partition functions for '
             'partition set {partition_set_name}'.format(partition_set_name=partition_set_def.name),
         ):
-            run_config = partition_set_def.environment_dict_for_partition(partition)
+            run_config = partition_set_def.run_config_for_partition(partition)
             tags = partition_set_def.tags_for_partition(partition)
             return ExternalPartitionData(name=partition.name, tags=tags, run_config=run_config)
     except PartitionScheduleExecutionError:
@@ -485,33 +484,30 @@ def _schedule_tick_state(instance, tick_data):
 @click.option('--schedule_name')
 def launch_scheduled_execution(output_file, schedule_name, **kwargs):
     with ipc_write_stream(output_file) as stream:
-        recon_repo = get_reconstructable_repository_from_origin_kwargs(kwargs)
-
         instance = DagsterInstance.get()
 
-        repo_def = recon_repo.get_definition()
-
-        if not repo_def.has_schedule_def(schedule_name):
-            raise DagsterInvariantViolationError(
-                'Schedule named "{sched}" not found in repository "{repo}"'.format(
-                    sched=schedule_name, repo=repo_def.name
-                )
-            )
-
+        recon_repo = get_reconstructable_repository_from_origin_kwargs(kwargs)
         schedule = recon_repo.get_reconstructable_schedule(schedule_name)
-        schedule_def = schedule.get_definition()
 
+        # open the tick scope before we call get_definition to make sure
+        # load errors are stored in DB
         with _schedule_tick_state(
             instance,
             ScheduleTickData(
                 schedule_origin_id=schedule.get_origin_id(),
-                schedule_name=schedule_def.name,
-                cron_schedule=schedule_def.cron_schedule,
+                schedule_name=schedule_name,
                 timestamp=time.time(),
+                cron_schedule=None,  # not yet loaded
                 status=ScheduleTickStatus.STARTED,
             ),
         ) as tick:
 
+            schedule_def = schedule.get_definition()
+
+            tick.update_with_status(
+                status=ScheduleTickStatus.STARTED, cron_schedule=schedule_def.cron_schedule,
+            )
+
             pipeline = recon_repo.get_reconstructable_pipeline(
                 schedule_def.pipeline_name
             ).subset_for_execution(schedule_def.solid_selection)
@@ -539,15 +535,15 @@ def _launch_scheduled_execution(instance, schedule_def, pipeline, tick, stream):
 
     errors = []
 
-    environment_dict = {}
+    run_config = {}
     schedule_tags = {}
     try:
         with user_code_error_boundary(
             ScheduleExecutionError,
-            lambda: 'Error occurred during the execution of environment_dict_fn for schedule '
+            lambda: 'Error occurred during the execution of run_config_fn for schedule '
             '{schedule_name}'.format(schedule_name=schedule_def.name),
         ):
-            environment_dict = schedule_def.get_environment_dict(schedule_context)
+            run_config = schedule_def.get_run_config(schedule_context)
     except DagsterUserCodeExecutionError:
         error_data = serializable_error_info_from_exc_info(sys.exc_info())
         errors.append(error_data)
@@ -569,10 +565,9 @@ def _launch_scheduled_execution(instance, schedule_def, pipeline, tick, stream):
 
     mode = schedule_def.mode
 
+    execution_plan_snapshot = None
     try:
-        execution_plan = create_execution_plan(
-            pipeline_def, environment_dict=environment_dict, mode=mode,
-        )
+        execution_plan = create_execution_plan(pipeline_def, run_config=run_config, mode=mode,)
         execution_plan_snapshot = snapshot_from_execution_plan(
             execution_plan, pipeline_def.get_pipeline_snapshot_id()
         )
@@ -584,7 +579,7 @@ def _launch_scheduled_execution(instance, schedule_def, pipeline, tick, stream):
     possibly_invalid_pipeline_run = instance.create_run(
         pipeline_name=schedule_def.pipeline_name,
         run_id=None,
-        environment_dict=environment_dict,
+        run_config=run_config,
         mode=mode,
         solids_to_execute=pipeline.solids_to_execute,
         step_keys_to_execute=None,
