@@ -104,7 +104,7 @@ class PipelineDefinition(IContainSolids):
             def apply_op(context, num):
                 return context.resources.op(num)
 
-            @resource(config=Int)
+            @resource(config_schema=Int)
             def adder_resource(init_context):
                 return lambda x: x + init_context.resource_config
 
@@ -118,7 +118,7 @@ class PipelineDefinition(IContainSolids):
 
             add_three_preset = PresetDefinition(
                 name='add_three_preset',
-                environment_dict={'resources': {'op': {'config': 3}}},
+                run_config={'resources': {'op': {'config': 3}}},
                 mode='add_mode',
             )
 
@@ -432,11 +432,17 @@ class PipelineDefinition(IContainSolids):
         return self.get_pipeline_index().pipeline_snapshot_id
 
     def get_pipeline_index(self):
-        from dagster.core.snap import PipelineSnapshot
+        from dagster.core.snap import (
+            PipelineSnapshot,
+            PipelineSnapshotWithID,
+        )
         from dagster.core.host_representation import PipelineIndex
 
         return PipelineIndex(
-            PipelineSnapshot.from_pipeline_def(self), self.get_parent_pipeline_snapshot()
+            PipelineSnapshotWithID.from_snapshot(PipelineSnapshot.from_pipeline_def(self)),
+            PipelineSnapshotWithID.from_snapshot(self.get_parent_pipeline_snapshot())
+            if self.parent_pipeline_def
+            else None,
         )
 
     def get_config_schema_snapshot(self):
@@ -477,6 +483,9 @@ class PipelineSubsetDefinition(PipelineDefinition):
     def get_parent_pipeline_snapshot(self):
         return self._parent_pipeline_def.get_pipeline_snapshot()
 
+    def get_parent_pipeline_snapshot_id(self):
+        return self._parent_pipeline_def.get_pipeline_snapshot_id()
+
     @property
     def is_subset_pipeline(self):
         return True
