@@ -12,14 +12,16 @@ from rx import Observable
 
 from dagster import check, seven
 from dagster.config import Field, Permissive
-from dagster.core.definitions.pipeline import ExecutionSelector, PipelineDefinition
+from dagster.core.definitions.pipeline import PipelineDefinition, PipelineSubsetDefinition
 from dagster.core.errors import (
     DagsterInvalidConfigError,
     DagsterInvariantViolationError,
     DagsterRunAlreadyExists,
     DagsterRunConflict,
 )
+from dagster.core.storage.migration.utils import upgrading_instance
 from dagster.core.storage.pipeline_run import PipelineRun, PipelineRunStatus, PipelineRunsFilter
+from dagster.core.utils import str_format_list
 from dagster.serdes import ConfigurableClass, whitelist_for_serdes
 from dagster.seven import get_current_datetime_in_utc
 from dagster.utils.merger import merge_dicts
@@ -32,6 +34,7 @@ from .ref import InstanceRef, compute_logs_directory
 # airflow ingestion logic (see: dagster_pipeline_factory.py). 'airflow_execution_date' stores the
 # 'execution_date' used in Airflow operator execution and 'is_airflow_ingest_pipeline' determines
 # whether 'airflow_execution_date' is needed.
+# https://github.com/dagster-io/dagster/issues/2403
 AIRFLOW_EXECUTION_DATE_STR = 'airflow_execution_date'
 IS_AIRFLOW_INGEST_PIPELINE_STR = 'is_airflow_ingest_pipeline'
 
@@ -198,7 +201,7 @@ class DagsterInstance:
             schedule_storage, 'schedule_storage', ScheduleStorage
         )
         self._scheduler = check.opt_inst_param(scheduler, 'scheduler', Scheduler)
-        self._run_launcher = check.opt_inst_param(run_launcher, 'run_launcher', RunLauncher)
+        self._run_launcher = check.inst_param(run_launcher, 'run_launcher', RunLauncher)
         self._dagit_settings = check.opt_dict_param(dagit_settings, 'dagit_settings')
         self._telemetry_settings = check.opt_dict_param(telemetry_settings, 'telemetry_settings')
 
@@ -210,10 +213,11 @@ class DagsterInstance:
 
     @staticmethod
     def ephemeral(tempdir=None):
+        from dagster.core.launcher.sync_in_memory_run_launcher import SyncInMemoryRunLauncher
         from dagster.core.storage.event_log import InMemoryEventLogStorage
         from dagster.core.storage.root import LocalArtifactStorage
         from dagster.core.storage.runs import InMemoryRunStorage
-        from dagster.core.storage.local_compute_log_manager import NoOpComputeLogManager
+        from dagster.core.storage.noop_compute_log_manager import NoOpComputeLogManager
 
         if tempdir is None:
             tempdir = DagsterInstance.temp_storage()
@@ -223,7 +227,8 @@ class DagsterInstance:
             local_artifact_storage=LocalArtifactStorage(tempdir),
             run_storage=InMemoryRunStorage(),
             event_storage=InMemoryEventLogStorage(),
-            compute_log_manager=NoOpComputeLogManager(compute_logs_directory(tempdir)),
+            compute_log_manager=NoOpComputeLogManager(),
+            run_launcher=SyncInMemoryRunLauncher(),
         )
 
     @staticmethod
@@ -389,11 +394,16 @@ class DagsterInstance:
             return dagster_telemetry_enabled_default
 
     def upgrade(self, print_fn=lambda _: None):
-        print_fn('Updating run storage...')
-        self._run_storage.upgrade()
+        with upgrading_instance(self):
 
-        print_fn('Updating event storage...')
-        self._event_storage.upgrade()
+            print_fn('Updating run storage...')
+            self._run_storage.upgrade()
+
+            print_fn('Updating event storage...')
+            self._event_storage.upgrade()
+
+            print_fn('Updating schedule storage...')
+            self._schedule_storage.upgrade()
 
     def dispose(self):
         self._run_storage.dispose()
@@ -410,6 +420,22 @@ class DagsterInstance:
     def has_pipeline_snapshot(self, snapshot_id):
         return self._run_storage.has_pipeline_snapshot(snapshot_id)
 
+    def get_historical_pipeline(self, snapshot_id):
+        from dagster.core.host_representation import HistoricalPipeline
+
+        snapshot = self._run_storage.get_pipeline_snapshot(snapshot_id)
+        parent_snapshot = (
+            self._run_storage.get_pipeline_snapshot(snapshot.lineage_snapshot.parent_snapshot_id)
+            if snapshot.lineage_snapshot
+            else None
+        )
+        return HistoricalPipeline(
+            self._run_storage.get_pipeline_snapshot(snapshot_id), snapshot_id, parent_snapshot
+        )
+
+    def has_historical_pipeline(self, snapshot_id):
+        return self._run_storage.has_pipeline_snapshot(snapshot_id)
+
     def get_execution_plan_snapshot(self, snapshot_id):
         return self._run_storage.get_execution_plan_snapshot(snapshot_id)
 
@@ -430,14 +456,15 @@ class DagsterInstance:
         pipeline_def,
         execution_plan=None,
         run_id=None,
-        environment_dict=None,
+        run_config=None,
         mode=None,
-        solid_subset=None,
+        solids_to_execute=None,
         step_keys_to_execute=None,
         status=None,
         tags=None,
         root_run_id=None,
         parent_run_id=None,
+        solid_selection=None,
     ):
         from dagster.core.execution.api import create_execution_plan
         from dagster.core.execution.plan.plan import ExecutionPlan
@@ -446,23 +473,46 @@ class DagsterInstance:
         check.inst_param(pipeline_def, 'pipeline_def', PipelineDefinition)
         check.opt_inst_param(execution_plan, 'execution_plan', ExecutionPlan)
 
-        if solid_subset:
-            pipeline_def = pipeline_def.build_sub_pipeline(solid_subset=solid_subset)
+        # note that solids_to_execute is required to execute the solid subset, which is the
+        # frozenset version of the previous solid_subset.
+        # solid_selection is not required and will not be converted to solids_to_execute here.
+        # i.e. this function doesn't handle solid queries.
+        # solid_selection is only used to pass the user queries further down.
+        check.opt_set_param(solids_to_execute, 'solids_to_execute', of_type=str)
+        check.opt_list_param(solid_selection, 'solid_selection', of_type=str)
+
+        if solids_to_execute:
+            if isinstance(pipeline_def, PipelineSubsetDefinition):
+                # for the case when pipeline_def is created by ExecutablePipeline or ExternalPipeline
+                check.invariant(
+                    solids_to_execute == pipeline_def.solids_to_execute,
+                    'Cannot create a PipelineRun from pipeline subset {pipeline_solids_to_execute} '
+                    'that conflicts with solids_to_execute arg {solids_to_execute}'.format(
+                        pipeline_solids_to_execute=str_format_list(pipeline_def.solids_to_execute),
+                        solids_to_execute=str_format_list(solids_to_execute),
+                    ),
+                )
+            else:
+                # for cases when `create_run_for_pipeline` is directly called
+                pipeline_def = pipeline_def.get_pipeline_subset_def(
+                    solids_to_execute=solids_to_execute
+                )
 
         if execution_plan is None:
             execution_plan = create_execution_plan(
                 pipeline_def,
-                environment_dict=environment_dict,
+                run_config=run_config,
                 mode=mode,
                 step_keys_to_execute=step_keys_to_execute,
             )
 
-        return self.get_or_create_run(
+        return self.create_run(
             pipeline_name=pipeline_def.name,
             run_id=run_id,
-            environment_dict=environment_dict,
+            run_config=run_config,
             mode=check.opt_str_param(mode, 'mode', default=pipeline_def.get_default_mode_name()),
-            solid_subset=solid_subset,
+            solid_selection=solid_selection,
+            solids_to_execute=solids_to_execute,
             step_keys_to_execute=step_keys_to_execute,
             status=status,
             tags=tags,
@@ -472,24 +522,28 @@ class DagsterInstance:
             execution_plan_snapshot=snapshot_from_execution_plan(
                 execution_plan, pipeline_def.get_pipeline_snapshot_id()
             ),
+            parent_pipeline_snapshot=pipeline_def.get_parent_pipeline_snapshot(),
         )
 
     def _construct_run_with_snapshots(
         self,
-        pipeline_name=None,
-        run_id=None,
-        environment_dict=None,
-        mode=None,
-        solid_subset=None,
-        step_keys_to_execute=None,
-        status=None,
-        tags=None,
-        root_run_id=None,
-        parent_run_id=None,
-        pipeline_snapshot=None,
-        execution_plan_snapshot=None,
+        pipeline_name,
+        run_id,
+        run_config,
+        mode,
+        solids_to_execute,
+        step_keys_to_execute,
+        status,
+        tags,
+        root_run_id,
+        parent_run_id,
+        pipeline_snapshot,
+        execution_plan_snapshot,
+        parent_pipeline_snapshot,
+        solid_selection=None,
     ):
 
+        # https://github.com/dagster-io/dagster/issues/2403
         if tags and IS_AIRFLOW_INGEST_PIPELINE_STR in tags:
             if AIRFLOW_EXECUTION_DATE_STR not in tags:
                 tags[AIRFLOW_EXECUTION_DATE_STR] = get_current_datetime_in_utc().isoformat()
@@ -497,9 +551,10 @@ class DagsterInstance:
         pipeline_run = PipelineRun(
             pipeline_name=pipeline_name,
             run_id=run_id,
-            environment_dict=environment_dict,
+            run_config=run_config,
             mode=mode,
-            solid_subset=solid_subset,
+            solid_selection=solid_selection,
+            solids_to_execute=solids_to_execute,
             step_keys_to_execute=step_keys_to_execute,
             status=status,
             tags=tags,
@@ -510,13 +565,29 @@ class DagsterInstance:
         if pipeline_snapshot is not None:
             from dagster.core.snap import create_pipeline_snapshot_id
 
-            pipeline_snapshot_id = create_pipeline_snapshot_id(pipeline_snapshot)
+            if pipeline_snapshot.lineage_snapshot:
+                if not self._run_storage.has_pipeline_snapshot(
+                    pipeline_snapshot.lineage_snapshot.parent_snapshot_id
+                ):
+                    check.invariant(
+                        create_pipeline_snapshot_id(parent_pipeline_snapshot)
+                        == pipeline_snapshot.lineage_snapshot.parent_snapshot_id,
+                        'Parent pipeline snapshot id out of sync with passed parent pipeline snapshot',
+                    )
 
+                    returned_pipeline_snapshot_id = self._run_storage.add_pipeline_snapshot(
+                        parent_pipeline_snapshot
+                    )
+                    check.invariant(
+                        pipeline_snapshot.lineage_snapshot.parent_snapshot_id
+                        == returned_pipeline_snapshot_id
+                    )
+
+            pipeline_snapshot_id = create_pipeline_snapshot_id(pipeline_snapshot)
             if not self._run_storage.has_pipeline_snapshot(pipeline_snapshot_id):
                 returned_pipeline_snapshot_id = self._run_storage.add_pipeline_snapshot(
                     pipeline_snapshot
                 )
-
                 check.invariant(pipeline_snapshot_id == returned_pipeline_snapshot_id)
 
             pipeline_run = pipeline_run.with_pipeline_snapshot_id(pipeline_snapshot_id)
@@ -552,30 +623,29 @@ class DagsterInstance:
 
     def create_run(
         self,
-        pipeline_name=None,
-        run_id=None,
-        environment_dict=None,
-        mode=None,
-        solid_subset=None,
-        step_keys_to_execute=None,
-        status=None,
-        tags=None,
-        root_run_id=None,
-        parent_run_id=None,
-        pipeline_snapshot=None,
-        execution_plan_snapshot=None,
+        pipeline_name,
+        run_id,
+        run_config,
+        mode,
+        solids_to_execute,
+        step_keys_to_execute,
+        status,
+        tags,
+        root_run_id,
+        parent_run_id,
+        pipeline_snapshot,
+        execution_plan_snapshot,
+        parent_pipeline_snapshot,
+        solid_selection=None,
     ):
 
-        # This logic is factored out since it's shared with `get_or_create_run`, which will soon be
-        # deleted. When it is deleted, the body of `_construct_run_with_snapshots` can be brought
-        # back into this function.
-        # https://github.com/dagster-io/dagster/issues/2412
         pipeline_run = self._construct_run_with_snapshots(
             pipeline_name=pipeline_name,
             run_id=run_id,
-            environment_dict=environment_dict,
+            run_config=run_config,
             mode=mode,
-            solid_subset=solid_subset,
+            solid_selection=solid_selection,
+            solids_to_execute=solids_to_execute,
             step_keys_to_execute=step_keys_to_execute,
             status=status,
             tags=tags,
@@ -583,44 +653,55 @@ class DagsterInstance:
             parent_run_id=parent_run_id,
             pipeline_snapshot=pipeline_snapshot,
             execution_plan_snapshot=execution_plan_snapshot,
+            parent_pipeline_snapshot=parent_pipeline_snapshot,
         )
         return self._run_storage.add_run(pipeline_run)
 
-    def get_or_create_run(
+    def register_managed_run(
         self,
-        pipeline_name=None,
-        run_id=None,
-        environment_dict=None,
-        mode=None,
-        solid_subset=None,
-        step_keys_to_execute=None,
-        status=None,
-        tags=None,
-        root_run_id=None,
-        parent_run_id=None,
-        pipeline_snapshot=None,
-        execution_plan_snapshot=None,
+        pipeline_name,
+        run_id,
+        run_config,
+        mode,
+        solids_to_execute,
+        step_keys_to_execute,
+        tags,
+        root_run_id,
+        parent_run_id,
+        pipeline_snapshot,
+        execution_plan_snapshot,
+        parent_pipeline_snapshot,
+        solid_selection=None,
     ):
-        # The last usage of this method is in dagster-airflow. When the usage is removed, this method
-        # should be deleted.
+        # The usage of this method is limited to dagster-airflow, specifically in Dagster
+        # Operators that are executed in Airflow. Because a common workflow in Airflow is to
+        # retry dags from arbitrary tasks, we need any node to be capable of creating a
+        # PipelineRun.
+        #
+        # The try-except DagsterRunAlreadyExists block handles the race when multiple "root" tasks
+        # simultaneously execute self._run_storage.add_run(pipeline_run). When this happens, only
+        # one task succeeds in creating the run, while the others get DagsterRunAlreadyExists
+        # error; at this point, the failed tasks try again to fetch the existing run.
         # https://github.com/dagster-io/dagster/issues/2412
 
         pipeline_run = self._construct_run_with_snapshots(
             pipeline_name=pipeline_name,
             run_id=run_id,
-            environment_dict=environment_dict,
+            run_config=run_config,
             mode=mode,
-            solid_subset=solid_subset,
+            solid_selection=solid_selection,
+            solids_to_execute=solids_to_execute,
             step_keys_to_execute=step_keys_to_execute,
-            status=status,
+            status=PipelineRunStatus.MANAGED,
             tags=tags,
             root_run_id=root_run_id,
             parent_run_id=parent_run_id,
             pipeline_snapshot=pipeline_snapshot,
             execution_plan_snapshot=execution_plan_snapshot,
+            parent_pipeline_snapshot=parent_pipeline_snapshot,
         )
 
-        if self.has_run(pipeline_run.run_id):
+        def get_run():
             candidate_run = self.get_run_by_id(pipeline_run.run_id)
 
             field_diff = _check_run_equality(pipeline_run, candidate_run)
@@ -634,7 +715,13 @@ class DagsterInstance:
                 )
             return candidate_run
 
-        return self._run_storage.add_run(pipeline_run)
+        if self.has_run(pipeline_run.run_id):
+            return get_run()
+
+        try:
+            return self._run_storage.add_run(pipeline_run)
+        except DagsterRunAlreadyExists:
+            return get_run()
 
     def add_run(self, pipeline_run):
         return self._run_storage.add_run(pipeline_run)
@@ -651,10 +738,22 @@ class DagsterInstance:
     def get_runs_count(self, filters=None):
         return self._run_storage.get_runs_count(filters)
 
+    def get_run_groups(self, filters=None, cursor=None, limit=None):
+        return self._run_storage.get_run_groups(filters=filters, cursor=cursor, limit=limit)
+
     def wipe(self):
         self._run_storage.wipe()
         self._event_storage.wipe()
 
+    def wipe_assets(self):
+        from dagster.core.storage.event_log.base import AssetAwareEventLogStorage
+
+        if not isinstance(self._event_storage, AssetAwareEventLogStorage):
+            return
+
+        for asset_key in self._event_storage.get_all_asset_keys():
+            self._event_storage.wipe_asset(asset_key)
+
     def delete_run(self, run_id):
         self._run_storage.delete_run(run_id)
         self._event_storage.delete_events(run_id)
@@ -772,100 +871,116 @@ class DagsterInstance:
 
     # Run launcher
 
-    def launch_run(self, run_id):
+    def launch_run(self, run_id, external_pipeline):
         '''Launch a pipeline run.
 
         This method delegates to the ``RunLauncher``, if any, configured on the instance, and will
         call its implementation of ``RunLauncher.launch_run()`` to begin the execution of the
         specified run. Runs should be created in the instance (e.g., by calling
-        ``DagsterInstance.get_or_create_run()``) *before* this method is called, and
+        ``DagsterInstance.create_run()``) *before* this method is called, and
         should be in the ``PipelineRunStatus.NOT_STARTED`` state.
 
         Args:
             run_id (str): The id of the run the launch.
         '''
         run = self.get_run_by_id(run_id)
-        return self._run_launcher.launch_run(self, run)
+        return self._run_launcher.launch_run(self, run, external_pipeline=external_pipeline)
 
     # Scheduler
 
-    def start_schedule(self, repository, schedule_name):
-        return self._scheduler.start_schedule(self, repository, schedule_name)
+    def reconcile_scheduler_state(self, external_repository):
+        return self._scheduler.reconcile_scheduler_state(self, external_repository)
 
-    def stop_schedule(self, repository, schedule_name):
-        return self._scheduler.stop_schedule(self, repository, schedule_name)
+    def start_schedule_and_update_storage_state(self, external_schedule):
+        return self._scheduler.start_schedule_and_update_storage_state(self, external_schedule)
 
-    def end_schedule(self, repository, schedule_name):
-        return self._scheduler.end_schedule(self, repository, schedule_name)
+    def stop_schedule_and_update_storage_state(self, schedule_origin_id):
+        return self._scheduler.stop_schedule_and_update_storage_state(self, schedule_origin_id)
+
+    def stop_schedule_and_delete_from_storage(self, schedule_origin_id):
+        return self._scheduler.stop_schedule_and_delete_from_storage(self, schedule_origin_id)
+
+    def running_schedule_count(self, schedule_origin_id):
+        if self._scheduler:
+            return self._scheduler.running_schedule_count(schedule_origin_id)
+        return 0
 
     def scheduler_debug_info(self):
-        output = ""
+        from dagster.core.scheduler import SchedulerDebugInfo, ScheduleStatus
 
-        title = "Scheduler Configuration"
-        output += "{title}\n{sep}\n{info}\n".format(
-            title=title,
-            sep="=" * len(title),
-            info=self.info_str_for_component('Scheduler', self.scheduler),
-        )
+        errors = []
 
-        title = "Scheduler Info"
-        output += "{title}\n{sep}\n{info}\n".format(
-            title=title, sep="=" * len(title), info=self.scheduler.debug_info(),
-        )
-
-        title = "Scheduler Storage Info"
-        schedule_info = self.all_schedules_info()
         schedules = []
-        for repository, schedule in schedule_info:
+        for schedule_state in self.all_stored_schedule_state():
+            if schedule_state.status == ScheduleStatus.RUNNING and not self.running_schedule_count(
+                schedule_state.schedule_origin_id
+            ):
+                errors.append(
+                    "Schedule {schedule_name} is set to be running, but the scheduler is not "
+                    "running the schedule.".format(schedule_name=schedule_state.name)
+                )
+            elif schedule_state.status == ScheduleStatus.STOPPED and self.running_schedule_count(
+                schedule_state.schedule_origin_id
+            ):
+                errors.append(
+                    "Schedule {schedule_name} is set to be stopped, but the scheduler is still running "
+                    "the schedule.".format(schedule_name=schedule_state.name)
+                )
+
+            if self.running_schedule_count(schedule_state.schedule_origin_id) > 1:
+                errors.append(
+                    "Duplicate jobs found: More than one job for schedule {schedule_name} are "
+                    "running on the scheduler.".format(schedule_name=schedule_state.name)
+                )
+
             schedule_info = {
-                schedule.name: {
-                    "status": schedule.status.value,
-                    "cron_schedule": schedule.cron_schedule,
-                    "python_path": schedule.python_path,
-                    "repository_name": repository,
-                    "repository_path": schedule.repository_path,
+                schedule_state.name: {
+                    "status": schedule_state.status.value,
+                    "cron_schedule": schedule_state.cron_schedule,
+                    "python_path": schedule_state.pipeline_origin.executable_path,
+                    "repository_pointer": schedule_state.pipeline_origin.get_repo_pointer().describe(),
+                    "schedule_origin_id": schedule_state.schedule_origin_id,
+                    "repository_origin_id": schedule_state.repository_origin_id,
                 }
             }
 
             schedules.append(yaml.safe_dump(schedule_info, default_flow_style=False))
 
-        output += "{title}\n{sep}\n{info}\n".format(
-            title=title, sep="=" * len(title), info="\n".join(schedules),
+        return SchedulerDebugInfo(
+            scheduler_config_info=self.info_str_for_component('Scheduler', self.scheduler),
+            scheduler_info=self.scheduler.debug_info(),
+            schedule_storage=schedules,
+            errors=errors,
         )
 
-        return output
-
     # Schedule Storage
 
-    def create_schedule_tick(self, repository, schedule_tick_data):
-        return self._schedule_storage.create_schedule_tick(repository, schedule_tick_data)
+    def create_schedule_tick(self, schedule_tick_data):
+        return self._schedule_storage.create_schedule_tick(schedule_tick_data)
 
-    def update_schedule_tick(self, repository, tick):
-        return self._schedule_storage.update_schedule_tick(repository, tick)
+    def update_schedule_tick(self, tick):
+        return self._schedule_storage.update_schedule_tick(tick)
 
-    def get_schedule_ticks_by_schedule(self, repository, schedule_name):
-        return self._schedule_storage.get_schedule_ticks_by_schedule(repository, schedule_name)
+    def get_schedule_ticks(self, schedule_origin_id):
+        return self._schedule_storage.get_schedule_ticks(schedule_origin_id)
 
-    def get_schedule_tick_stats_by_schedule(self, repository, schedule_name):
-        return self._schedule_storage.get_schedule_tick_stats_by_schedule(repository, schedule_name)
+    def get_schedule_tick_stats(self, schedule_origin_id):
+        return self._schedule_storage.get_schedule_tick_stats(schedule_origin_id)
 
-    def all_schedules_info(self):
-        return self._schedule_storage.all_schedules_info()
+    def all_stored_schedule_state(self, repository_origin_id=None):
+        return self._schedule_storage.all_stored_schedule_state(repository_origin_id)
 
-    def all_schedules(self, repository=None):
-        return self._schedule_storage.all_schedules(repository)
+    def get_schedule_state(self, schedule_origin_id):
+        return self._schedule_storage.get_schedule_state(schedule_origin_id)
 
-    def get_schedule_by_name(self, repository, schedule_name):
-        return self._schedule_storage.get_schedule_by_name(repository, schedule_name)
+    def add_schedule_state(self, schedule_state):
+        return self._schedule_storage.add_schedule_state(schedule_state)
 
-    def add_schedule(self, repository, schedule):
-        return self._schedule_storage.add_schedule(repository, schedule)
+    def update_schedule_state(self, schedule_state):
+        return self._schedule_storage.update_schedule_state(schedule_state)
 
-    def update_schedule(self, repository, schedule):
-        return self._schedule_storage.update_schedule(repository, schedule)
-
-    def delete_schedule(self, repository, schedule):
-        return self._schedule_storage.delete_schedule(repository, schedule)
+    def delete_schedule_state(self, schedule_origin_id):
+        return self._schedule_storage.delete_schedule_state(schedule_origin_id)
 
     def wipe_all_schedules(self):
         if self._scheduler:
@@ -873,5 +988,5 @@ class DagsterInstance:
 
         self._schedule_storage.wipe()
 
-    def log_path_for_schedule(self, repository, schedule_name):
-        return self._scheduler.get_log_path(self, repository, schedule_name)
+    def logs_path_for_schedule(self, schedule_origin_id):
+        return self._scheduler.get_logs_path(self, schedule_origin_id)
